c	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c	+							+
c	+  This subroutine defines the basic functions of	+
c	+  plotting. The results will be a postscript file.	+
c	+							+
c	+  The program is written by Jess U Jorgensen (DMI).	+
c	+  More fonts are included; line thickness are modified +
c	+  and some small modifications are made by Hans X.-Y. 	+
c	+  Huang (DMI).						+
c	+							+
c	+  The program has been tested with MetgraF, XYPLOT on 	+
c	+  CONVEX, SUN, SGI, DEC, ...				+
c	+							+
c	+  Please send your comments and modifications to: 	+
c	+          juj@dmicvx.dmi.min.dk			+
c	+	   xyh@dmicvx.dmi.min.dk                        +
c	+							+
c	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      	subroutine plotex(a,landscape)
      	parameter (mcol=30)
      	character*(*) a
        logical landscape
      	character*(*) symb,font
      	character*12 col(mcol)
        character*50 curfont
      	real xc(*),yc(*)
      	save curfont,lenfon,lu,lc,lw,ihold,linold
      	isx(z)=nint((900./7.5)*z)  ! xscale between cm and inch*1000
      	isy(z)=nint((900./7.5)*z)  ! yscale between cm and inch*1000
      	ifs(z)=nint(240*z)        ! font scale
c
c       initial values
      	lu = 9
      	lw = 1
        lc = 1
        ihold = 0
        lenfon = 5
        curfont ='Times'
c 
c 	================= open postscript file ======================
c
      	nn=len(a)
      	if ( nn.gt.1.and.nn.lt.20) then
      		open(lu,file=a,status='unknown')
      	else
      		open(lu,file='POST',status='unknown')
      	endif
      	write(lu,'(a4)') '%!PS'
      	write(lu,'(a)')  '0.25 0.25 scale'
        if(landscape) then
		write(lu,*) '90 rotate'
		write(lu,*) '0 -2800 translate'
        endif
      	write(lu,'(a)') '/initgraphics'
c
c	======================== line ===========================
c
      	write(lu,'(a)') '/L {lineto} def'
      	write(lu,'(a)') '/M {moveto} def'
      	write(lu,'(a)') '/l {newpath moveto lineto stroke } def'
c
c	                     line types 
c
      	write(lu,'(a)') '/l1 [       ] def'
      	write(lu,'(a)') '/l2 [ 8 8    ] def'
      	write(lu,'(a)') '/l3 [ 16 8   ] def'
      	write(lu,'(a)') '/l4 [ 20 8 8 ] def'
      	write(lu,'(a)') '/l5 [ 8 4   ] def'
      	write(lu,'(a)') '/l6 [ 24 4 4 24 ] def'
      	write(lu,'(a)') '/l7 [ 4 40 4  ] def'
      	write(lu,'(a)') '/T  { 0 setdash } def'
c 
      	do i=1,6
      		write(lu,'(a2,i1,a4,i1,a8)') 
     1               '/t',i,' { l',i,' T } def'
      	enddo
c 
c 	================== colour definition  =======================
c               
c  	color table 01      black
c      	            02 - 16 normal spectrum
c                   17 - 23 yellow-green
c                   24 - 29 sharp contrast
c                   30      white
c
     	col( 1) = '0    0    0 '  ! black
     	col( 2) = '0.3  0    0 '  ! red
     	col( 3) = '0.6  0    0 '  ! red
     	col( 4) = '1    0    0 '  ! red-orange
     	col( 5) = '1    0.4  0 '  ! orange
     	col( 6) = '1    1    0 '  ! yellow
     	col( 7) = '0.6 0.6   0 '  ! dark yellow
     	col( 8) = '0.2 0.9   0 '  ! yellow
     	col( 9) = '0.1 0.5 0.3 '  ! green-yellow
     	col(10) = '0   0.5 0.5 '  ! light green
     	col(11) = '0   0.4 0.6 '  ! green
     	col(12) = '0   0.3 0.7 '  ! dark green
     	col(13) = '0   0.2 0.8 '  ! green blue
     	col(14) = '0   0     1 '  ! blue
     	col(15) = '0   0   0.6 '  ! dark blue
     	col(16) = '0   0   0.3 '  ! very dark blue
     	col(17) = '0.0 0.4  0.0'  !
     	col(18) = '0.1 0.5  0.1'  !
     	col(19) = '0.2 0.6  0.2'  !
     	col(20) = '0.4 0.7  0.3'  !
     	col(21) = '0.6 0.8  0.4'  !
     	col(22) = '0.8 0.9  0.5'  !
     	col(23) = '1.0 1.0  0.6'  !
     	col(24) = '1   0     0 '  ! red
     	col(25) = '1   1     0 '  ! yellow
     	col(26) = '0   1     0 '  ! green
     	col(27) = '0   0     1 '  ! blue
     	col(28) = '0.5 0   0.5 '  ! purple
     	col(29) = '0.5 0.5 0.5 '  ! grey
     	col(30) = '1.0 1.0 1.0 '  ! white
c
c			colour switch
c
      	write(lu,'(a)') '/Color  true  def'
c
c			set coulour
c
      	do i=1,mcol
      		write(lu,'(a2,i2.2,a4,a12,a6)') 
     1               '/C',i,' { ',col(i),' } def'
      	enddo
c
c		       color or gray shading
c
      	write(lu,'(2a)') '/g { Color {setrgbcolor pop }',
     +  ' { pop pop pop 16 mod 1 sub 16 div setgray }ifelse } def'
c
c                   linetype or full color line
c
      	write(lu,'(2a)') '/c { Color { setrgbcolor [] 0 setdash pop}',
     +	' { pop pop pop 0 setdash } ifelse } def'
c
c 		color line or monochrome line 
c		( 1 in l1 has to be mod(l1,MLINE)+1)
c
      	do i=1,mcol
      		write(lu,'(a2,i2.2,a4,i1,a2,i2.2,a8)') '/c',i,
     +              ' { l',mod(i-1,6)+1,' C',i,' c } def'
      	enddo
c
c		color or gray scale 
c		( 1 should be mod(1-1,mcol)+1)
c
      	do i=1,mcol
      		write(lu,'(a2,i2.2,a4,i2,a2,i2.2,a8)') '/g',i,' {  ',
     +                         mod(i-1,mcol)+1,' C',i,' g } def'
      	enddo
c
c	=================== fonts ===============================
c
c			 Font Times
c
      	write(lu,'(2a)') '/FontTimes { /Times-Roman findfont ',
     + 'sf scalefont setfont moveto show } def'
c
      	write(lu,'(a)') 
     1 '/FontTimesR {/Times-Roman findfont sf scalefont setfont'
      	write(lu,'(a)') 
     1  ' translate rotate  0 0 FontTimes rotate translate } def'
c
c			 Font Helvetica
c
      	write(lu,'(2a)') '/FontHelvetica { /Helvetica findfont ',
     + 'sf scalefont setfont moveto show } def'
c
      	write(lu,'(a)') 
     1  '/FontHelveticaR {/Helvetica findfont sf scalefont setfont'
      	write(lu,'(a)') 
     1  ' translate rotate  0 0 FontHelvetica rotate translate } def'
c
c			 Font Symbol
c
      	write(lu,'(2a)') '/FontSymbol { /Symbol findfont ',
     1 'sf scalefont setfont moveto show } def'
c
      	write(lu,'(a)') 
     1 '/FontSymbolR {/Symbol findfont sf scalefont setfont'
      	write(lu,'(a)') 
     1  ' translate rotate  0 0 FontSymbol rotate translate } def'
c
c	====================== hashing ==========================
c
      	write(lu,'(a)') '/h { setlinewidth '
      	write(lu,'(a)') '/pos ymin def'
      	write(lu,'(a)') '/step step 5 mul def'
      	write(lu,'(a)') 'ymin step ymax { newpath '
      	write(lu,'(a)') '/pos pos step add def '
      	write(lu,'(a)') 'xmin pos moveto xmax pos lineto stroke } for '
      	write(lu,'(a)') '   } def '
      	write(lu,'(a)') '/v { setlinewidth '
      	write(lu,'(a)') '/pos xmin def'
      	write(lu,'(a)') '/step step 5 mul def'
      	write(lu,'(a)') 'xmin step xmax { newpath '
      	write(lu,'(a)') '/pos pos step add def'
      	write(lu,'(a)') 'pos ymin moveto pos ymax lineto stroke } for'
      	write(lu,'(a)') '   } def '
      	write(lu,'(a)') '/h1 { 3 h } def'
      	write(lu,'(a)') '/h2 { 6 h } def'
      	write(lu,'(a)') '/h3 { 3 v } def'
      	write(lu,'(a)') '/h4 { 6 v } def'
c
c	================ initialize line and text ===================
c
      	write(lu,'(a)') '0.1 setlinewidth c01'
       	write(lu,'(a4,i3,a5)')'/sf ',20,' def '
c
c	================== end of initialization  ===================
c
      	return
c
c 	================= write characters ==========================
c
      	entry symbex(x,y,hgt,symb,ang,nsymb)
       	irot=nint(ang)
       	ix=isx(x)
       	iy=isy(y)
       	ihgt=ifs(hgt)
       	if ( ihgt.eq.0 ) return 
       	if ( ihgt.ne.ihold ) write(lu,'(a4,i3,a5)')'/sf ',ihgt,' def '
       	ihold=ihgt
       	if ( ang.eq.0.) then
       		write(lu,'(a2,a,a2,2i6,2a)') 
     +          ' (',symb(1:nsymb),') ',ix,iy,' Font',curfont(1:lenfon)
       	else
       		write(lu,'(3i6,a2,a,a,3i6,3a)') 
     +          -ix,-iy,-irot,' (',symb(1:nsymb),') ',
     +          irot,ix,iy,' Font',curfont(1:lenfon),'R'
       	endif
      	return
c
c 	==================== fill an area ==========================
c
      	entry pfill(xc,yc,nc,ncol,nshade)
c
c  	ncol   values 1-16
c  	nshade values 0-16
c                0 ordinary fill
c                mod(n-1,4)+1  linewidth and shape   _____ |||| 
c                (n-1)/4+1     linetype      _____ --- __-- etc
c
       	if ( nshade.lt.0 ) return
       	xmin=xc(1)
       	xmax=xc(1)
       	ymin=yc(1)
       	ymax=yc(1)
       	if ( nc.ne.lc ) write (lu,'(a1,i2.2)') 'g',ncol
       	write(lu,'(a)') 'newpath'
       	write(lu,'(2i6,a)') isx(xc(1)),isy(yc(1)),' M'
       	do i=1,nc
       		if ( xc(i) .gt. xmax) then
        		xmax=xc(i)
       		elseif ( xc(i).lt.xmin) then
        		xmin=xc(i)
       		endif
       		if ( yc(i) .gt. ymax) then
        		ymax=yc(i)
       		elseif ( yc(i).lt.ymin) then
        		ymin=yc(i)
       		endif
       		write(lu,'(2i6,a)') isx(xc(i)),isy(yc(i)),' L'
       	enddo
       	if ( xc(nc).ne.xc(1).or.yc(nc).ne.yc(1)) then
       		write(lu,'(2i6,a)') isx(xc(1)),isy(yc(1)),' L'
       	endif
       	write(lu,'(a)') 'closepath'
       	if (nshade.eq.0) then
       		write(lu,'(a)') 'fill'
       	else
c
c 	write the variables used for hashing 
c
       		write(lu,'(a,i4,a)') 'clear /xmin ',isx(xmin),' def'
       		write(lu,'(a,i4,a)') '/xmax ',isx(xmax),' def'
       		write(lu,'(a,i4,a)') '/ymin ',isy(ymin),' def'
       		write(lu,'(a,i4,a)') '/ymax ',isy(ymax),' def'
       		write(lu,'(a,i4,a)') '/step ',mod(nshade-1,2)+1,' def'
       		write(lu,'(a1,i1)')  't',mod((nshade-1)/4,6)+1
       		write(lu,'(a12,i1,a)') 'gsave clip h',mod(nshade-1,4)+1,
     +                                 ' grestore'
       		write(lu,'(f4.1,a)')  float(lw)/10.,' setlinewidth'
       	endif
       	if ( nc.ne.lc.and.lc.gt.0) write (lu,'(a1,i2.2)') 'g',lc

       	return
c
c	====================== change font =============================
c
        entry newfon(font)
        do i = 1,130
        if(font(i:i).eq.' ') then
	      lenfon=i-1
	      go to 10
        endif
        enddo
   10   curfont=font(1:lenfon)
        return
c
c 	============= reset line thickness and color change ==========
c
      	entry newpex(ithick,icolor)
      	if ( ithick.eq.lw.and.icolor.eq.lc) return
      	lw=ithick
      	lc=max(1,icolor)
      	write(lu,'(f4.1,a15,i2.2)') float(lw)/10.,
     1		' setlinewidth c',mod(lc-1,mcol)+1
      	return
c
c 	================ draw a line between two points ==============
c
      	entry vectex(x1,y1,x2,y2,jtyp)
      	lintyp=mod(jtyp-1,6)+1
      	if ( linold.ne.lintyp) write(lu,'(a1,i1)') 't',lintyp
      	write(lu,'(4i6,a)') isx(x2),isy(y2),isx(x1),isy(y1),' l'
      	linold=lintyp
      	return
c
c 	================ draw a line with n points ==================
c
      	entry linex(xc,yc,n,jtyp)
          
      	lintyp=mod(jtyp-1,6)+1
      	if ( linold.ne.lintyp ) write(lu,'(a1,i1)') 't',lintyp
      	write(lu,'(a,2i6,a)') 'newpath ',isx(xc(1)),isy(yc(1)),' M'
      	do i=2,n
      		write(lu,'(2i6,a)') isx(xc(i)),isy(yc(i)),' L'
      	enddo
      	write(lu,'(a)') 'stroke'
      	linold=lintyp
      	return
c
c 	================ close the postscript file ==================
c
      	entry endex
      	print*, 'closing POST in endex'
      	write(lu,'(3a)') 'showpage ',char(10),'/initgraphics'
      	close(lu)
      	end
