
c-----------------------------------------------------------------------------
c
c   contouring program isolin
c
c   the fortran-call for isolin is as follows:
c
c     call isolin(z,x,y,m,n,dz,zlow,zupp,
c    x        nsmut,ltype,mrkfrq,mrkdec,hgtmrk,frame,
c    x        igrid,xmin,xmax,ymin,ymax,
c    x        maxvec,xp,yp,lin)
c
c   where
c
c    variable    type       content
c    --------    ----       -------
c    z(m,n)    r      grid-point-field to be plotted
c    x(m,n)    r      paper/screen x-coordinates of the gridpoints
c    y(m,n)    r      paper/screen y-coordinates of the gridpoints
c    m         i      x-dimension
c    n         i      y-dimension
c    dz        r      increment of z-values between isolines
c    zlow      r      z-value of first isoline
c    zupp      r      z-value of last isoline
c    nsmut     i      number of iterations in smoothing the coordinates
c                     of the isolines
c    ltype     i      type of line:
c                     1 - full,
c                     2 - dashed,
c                     3 - dotted,
c                     4 - full for values.ge.0.0 and dashed for values.lt.0.0
c                     5 - as ltype=4 but no 0-line is drawn
c    mrkfrq    i      frequency for marking the isolines ( 1=every,
c                     2=each 2nd, a.s.o. )
c    mrkdec    i      number of decimals in the marking of the lines
c                     (0 means marking with integers)
c    hgtmrk    r      height of the digits of the line-markers
c    frame     l      indicator for plotting frame around the map
c                     (xmin,ymin)-(xmax,ymin)- a.s.o.
c                     (.t. gives frame, .f. no frame)
c    igrid     i      plot of grid (0=no grid,1=grid boundaries,
c                     (2=mesh through all grid-points)
c    xmin      r      minimum paper/screen x -coordinate to be plotted
c    xmax      r      maximum    -"-             -"-
c    ymin      r      minimum    -"-       y -   -"-
c    ymax      r      maximum    -"-             -"-
c    maxvec    i      max. number of vectors in an isoline to be
c                     plotted in one step with one mark, note that one
c                     grid-length will correspond to approximately
c                     3-5 vectors
c    xp(maxvec)    r  working array of length maxvec
c    yp(maxvec)    r  working array of length maxvec
c    lin(4*(m+2)*(n+2))
c         string*1    working array of length (m+2)*(n+2)*4
c
c    note that unit of all coordinates are in the unit defined by the
c    window-statement. (normally cm)
c
c----------------------------------------------------------------------------
c
      subroutine isolin(z,x,y,m,n,dz,zlow,zupp,nsmut,ltype,
     x                  mrkfrq,mrkdec,hgtmrk,frame,
     x                  igrid,xmin,xmax,ymin,ymax,
     x                  maxvec,xp,yp,lin)
c
      implicit none
c
      integer m,n
      real z(m,n),x(m,n),y(m,n)
      real dz,zlow,zupp,hgtmrk
      integer nsmut,ltype,mrkfrq,mrkdec,igrid,maxvec
      real xmin,xmax,ymin,ymax
      real xp(maxvec),yp(maxvec)
      character*1 lin(4,0:m+1,0:n+1)
      logical frame
c
c---------------------------------------------------------------------------
c
      real zmin,zmax,zval
      integer i,j,nline
c
c---------------------------------------------------------------------------
c
      call iso2(x,y,m,n,frame,
     x          igrid,xmin,xmax,ymin,ymax,
     x          maxvec,xp,yp,lin)
c
c
      zmin = +1.e+10
      zmax = -1.e+10
      do 90 j=1,n
         do 90 i=1,m
            if( lin(4,i,j).eq.'1' ) then
               zmin=min(zmin,z(i,j))
               zmax=max(zmax,z(i,j))
            endif
   90 continue
c
      if ( zmin.le.zlow ) then
         zmin = zlow
      else
         zmin = zlow +
     x     float( int( (zmin-zlow)/dz ) + 1 )*dz
      endif
      if( zmax.ge.zupp ) then
         zmax = zupp
      else
         zmax = zlow +
     x     float( int( (zmax-zlow)/dz ) )*dz
      endif
c
      zval = zmin
c
      nline = 0
c
   95 continue
c
c
      call iso1(zval,nline,z,x,y,m,n,nsmut,ltype,
     x          mrkfrq,mrkdec,hgtmrk,
     x          xmin,xmax,ymin,ymax,
     x          maxvec,xp,yp,lin)
c
c
      nline = nline + 1
c
      zval = zval + dz
c
      if( zval.le.zmax ) goto 95
c
      return
      end

