      subroutine scoast
c
c****************************************************************
c**                                                            **
c**  generate coastline data file from card images             **
c**  ---------------------------------------------             **
c**  purpose                                                   **
c**    read the card image file of coastline coordinates       **
c**    (format(14i5)) and convert them to requested geometry.  **
c**    geometry conversion is from reg. lat-lon to             **
c**    stereographic or lat-lon with shifted pole or           **
c**    "rikets koordinater"                                    **
c**    selection and required parameters are requested         **
c**    from input                                              **
c**                                                            **
c**  input                                                     **
c**    from unit 1      : card images of coastline coordinates **
c**                        under format(14i5).                 **
c**                                                            **
c**  output                                                    **
c**    to unit 2        : output coastline data, as required   **
c**                       by subroutine coast                  **
c**                       (unformatted)                        **
c**                                                            **
c****************************************************************
c
c
      integer lcard
      dimension lcard(14)
      character*128 filnam
      real xout(1000),yout(1000)
      character*1 cinput
      character*1 clbase
c
#include "comrep.inc"
#include "comdef.inc"
c
      write(6,*)'enter required map projection (0,1,5,10 or 98)!'
      read(5,*)maprep
      if( maprep.eq.5 ) then
         write(6,*)'enter latitude for stereographic projection'
         read(5,*)prolat
         write(6,*)'enter longitude for stereographic projection'
         read(5,*)prolon
      else if( maprep.eq.10 ) then
         write(6,*)'enter latitude of shifted sourth pole'
         read(5,*)alatnp
         write(6,*)'enter longitude of shifted sourth pole'
         read(5,*)prolon
      else if( maprep.eq.1 ) then
         write(6,*)'enter latitude of mercator projection'
         read(5,*)prolat
      else if( maprep.ne.0 .and. maprep.ne.98) then
         write(6,*)'map projection ',maprep,' not introduced'
         stop
      endif
c
      write(6,*)'enter input coastline file (g,n or s)!'
      read(5,*)cinput
c
      goto 50
      entry mkcoast(clbase)
      cinput=clbase 
   50 continue
      call clname(cinput,filnam)
c
      write(6,*)'coasline file is ',filnam
c
      open(unit=2,file=filnam,form='unformatted')
c
      call clorig(cinput,filnam)
c
      write(6,*)'the original file is ',filnam
      open(unit=1,file=filnam,form='formatted')
c
      pi     = 4.*atan(1.)
      rearth = 6.371e+06
      zradi = pi/180.
c
c------------------------------------------------------------------------------
c
c         map projection is polarstereographic
c

      if( maprep.eq.5 ) then
         zprola = prolat*pi/180.
         if( prolat.ge.0. ) then
            zrlen = ( 1. + sin(zprola))*rearth
         else
            zrlen = ( 1. - sin(zprola))*rearth
         endif
c
c        map projection is mercator
c
      else if( maprep.eq.1 ) then
         zmult = rearth*cos(prolat*zradi)
c
      endif
c
c-----------------------------------------------------------------------------
c
c           loop over all input records
c
      knvec = 1
      numvec = 0
c
c
      write(6,*) 'start to read ...'
 1000 read(1,1010)(lcard(i),i=1,14)
 1010 format(14i5)
c
c          loop through card.
c          pick up two coordinates each time. the first is longitude
c          and the second is latitude.
c
      do 1100 i=1,14,2
      ilon=lcard(i)
      ilat=lcard(i+1)
c
c             check for end
c
      if(ilon.eq.32767) goto 2000
c
c             check for start of new vector
c
      if( ilat.eq.0 .and. ilon.eq.0 ) goto 1050
c
c             here is the unpacking algorithm
c
      zlon =float(ilon)/100.-180.
      zlat =float(ilat)/100.- 90.
c
c            coordinate transformation
c
      if( maprep.eq.0 ) then
         y = zlat
         x = zlon
      else if( maprep.eq.10)then
         zlon = zlon - prolon
         y = fiprim(zlat,zlon,alatnp)
         x = alampr(zlat,zlon,alatnp)
      else if( maprep.eq.1)then
         if( zlon.gt.180. ) zlon = zlon - 360.
         x = zmult*zlon*zradi
         if( zlat.gt.89.9 ) zlat = 89.9
         if( zlat.lt.-89.9 ) zlat = -89.9
         y = zmult*alog( (1.+sin(zlat*zradi))/cos(zlat*zradi) )
      else if( maprep.eq.98 )then
         call grerik(zlat,zlon,y,x)
      else
         zlon = zlon - prolon
         if( prolat.gt.0. ) then
            if( zlat.lt.-20. ) goto 1050
            zlat = zlat*pi/180.
            zlon = zlon*pi/180.
            zr = zrlen*cos(zlat)/(1.+sin(zlat))
         else
            if( zlat.gt.+20. ) goto 1050
            zlat = zlat*pi/180.
            zlon = -zlon*pi/180.
            zr = zrlen*cos(zlat)/(1.-sin(zlat))
         endif
         x =  zr*sin(zlon)
         y = -zr*cos(zlon)
      endif
c
      numvec = numvec + 1
      xout(knvec+numvec) = x
      yout(knvec+numvec) = y
      if( (knvec+numvec).eq.997 ) then
         xout(knvec) = float(numvec)
         yout(knvec) = 0.
         xout(1000) = -1.
         yout(1000) = -1.
         write(2) xout,yout
         knvec = 1
         numvec = 1
         xout(2) = x
         yout(2) = y
      endif
      goto 1100
c
 1050 if( numvec.gt.0 ) then
         xout(knvec) = float(numvec)
         yout(knvec) = 0.
         knvec = knvec + numvec + 3
         numvec = 0
         if( knvec.gt.995 ) then
            xout(knvec) = -1.
            yout(knvec) = -1.
            write(2) xout,yout
            knvec = 1
         endif
      endif
c
 1100 continue
c
      goto 1000
c
c
 2000 if( numvec.gt.0 ) then
         xout(knvec) = float(numvec)
         yout(knvec) = 0.
         knvec = knvec + numvec + 3
      endif
      xout(knvec) = -1.
      yout(knvec) = -1.
      write(2) xout,yout
      close(2)
      close(1)
      end
