
c---------------------------------------------------------------------------
c    type:        subroutine xyplot
c    func:        sets up the geometry according to map projection.
c    system:      metgraf
c    author:      nils gustafsson mf
c    date:
c
c    call xyplot(ndim,xgrid,ygrid,xmin,xmax,ymin,ymax)
c
c    where
c
c    variable	 dim   type (i/o/w)	 content
c    --------	 ---   ---- ------    -------
c    ndim              i   (i)   dimension of grid data array xgrid, ygrid.
c    xgrid,ygrid(ndim) r   (o)   coordinates for grid points in cm.
c    xmin              r   (i)   minimum paper/screen x -coord to be plotted
c    xmax              r   (i)   maximum    -"-             -"-
c    ymin              r   (i)   minimum    -"-       y -   -"-
c    ymax              r   (i)   maximum    -"-             -"-
c
c
c    entry points:       none
c    called subroutines: kulan, grerik, rikgre
c
c---------------------------------------------------------------------------
      subroutine xyplot(ndim,xgrid,ygrid,xmin,xmax,ymin,ymax)
      integer ndim
      real xgrid(ndim),ygrid(ndim)

#include "comrep.inc"
#include "comdef.inc"
#include "comlog.inc"

      pi     = 4.*atan(1.)
      zradi  = pi/180.
      eps    = 5.e-4
      epspol = pi/2. - eps
      rearth = 6.371e+06

c*****************************************************************************
c
c    input field is in regular lat/long grid or
c                           gaussian lat/regular long grid
c
      if( irep.eq.0 .or. irep.eq.04 .or.
     x     (irep.eq.10 .and. alatnp.gt.epspol/zradi) ) then

c-----------------------------------------------------------------------------
c
c    map projection is regular lat/long

         if( maprep.eq.0 ) then

            zfac = zradi*rearth/(scale*1.e+6)*100.0

            if( ltotal ) then
               zlomin = alonw
               if( irep.eq.04 ) then
                  zlamin = gaulat(jsgaus)
               else
                  zlamin = alats
               endif
               xmin = 1.e-5
               ymin = 1.e-5
               xmax = (alone-alonw)*zfac + 2.e-5
               if( irep.eq.04 ) then
                  ymax =
     x               (gaulat(jsgaus-nlat+1)-gaulat(jsgaus))*zfac
     x               + 2.e-5
               else
                  ymax = (alatn-alats)*zfac + 2.e-5
               endif
               zlolim = zlomin - dlamda - eps

            else
c
               zlamin = latmid - 0.5*dymap/zfac
               zlomin = lonmid - 0.5*dxmap/zfac
               zlomax = lonmid + 0.5*dxmap/zfac
               if( zlomin.lt.-180. ) then
                  zlomin = zlomin + 360.
                  zlolim = lonmid + 180.
                  if( zlolim.gt.180. ) zlolim = zlolim - 360.
               else if( zlomax.gt.180. ) then
                  zlolim = lonmid + 180.
                  if( zlolim.gt.180. ) zlolim = zlolim - 360.
               else
                  zlolim = -180.0 - eps
               endif
               xmin = 1.e-5
               ymin = 1.e-5
               xmax = dxmap + 2.e-5
               ymax = dymap + 2.e-5
            endif
c
            if( irep.eq.04 ) then
               zyadd = -zlamin*zfac + 1.5e-5
            else
               zyadd = ( alats - dtheta - zlamin)*zfac
     x      +  1.5e-5
            endif
c
            if( irep.eq.04 ) then
               zymult = zfac
            else
               zymult = dtheta*zfac
            endif
c
            k = 1
            do 20 j=1,nlat
               if( irep.eq.04 ) then
                  zy = gaulat(jsgaus-j+1)*zymult + zyadd
               else
                  zy = j*zymult + zyadd
               endif
               do 20 i=1,nlon
                  zlon = alonw + float(i-1)*dlamda
                  if( zlon.lt.zlolim ) zlon = zlon + 360.
                  xgrid(k) = (zlon-zlomin)*zfac
                  ygrid(k) = zy
   20       k = k + 1
            return

c-----------------------------------------------------------------------------
c
c    map projection is mercator
c
         else if( maprep.eq.01 ) then
c
            zprola = prolat*zradi
            zmult = rearth*cos(zprola)/(scale*1.e+6)*100.
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi
            if( zlonm.gt.pi ) zlonm = zlonm - 2.*pi
            zxor = zmult*zlonm - 0.5*dxmap
            zyor = zmult*alog( (1.+sin(zlatm))/cos(zlatm) )
     +        - 0.5*dymap
            if( zxor/zmult.lt.-pi ) then
               zlonm = lonmid*zradi + 2.*pi
               zxor = zmult*zlonm - 0.5*dxmap
            endif
c
            k = 1
            do 26 j=1,nlat
               if( irep.eq.04 ) then
                  zlatg = gaulat(jsgaus-j+1)*zradi
               else
                  zlatg = (alats + (j-1)*dtheta)*zradi
               endif
               if( zlatg.lt.-epspol ) zlatg = -epspol
               if( zlatg.gt.epspol ) zlatg = epspol
               zy = zmult*alog( (1.+sin(zlatg))/cos(zlatg) ) - zyor
               zlongm = zxor/zmult - dlamda*zradi
               do 24 i=1,nlon
                  zlong = (alonw + (i-1)*dlamda)*zradi
                  if( zlong.lt.zlongm ) zlong = zlong + 2.*pi
                  xgrid(k) = zmult*zlong - zxor
                  ygrid(k) = zy
   24          k = k + 1
   26       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return

c-----------------------------------------------------------------------------
c
c    map projection is polarstereographic
c
         else if( maprep.eq.05 ) then
c
            zprola = prolat*zradi
            zprolo = prolon*zradi
            if( prolat.ge.0. ) then
               zrlen =
     +            ( 1. + sin(zprola))*rearth/(scale*1.e+6)*100.
            else
               zrlen =
     +            ( 1. - sin(zprola))*rearth/(scale*1.e+6)*100.
            endif
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi - zprolo
            if( prolat.ge.0. ) then
               zrmid = zrlen*cos(zlatm)/(1.+sin(zlatm))
            else
               zrmid = zrlen*cos(zlatm)/(1.-sin(zlatm))
               zlonm = - zlonm
            endif
            zyor = - zrmid*cos(zlonm) - 0.5*dymap
            zxor =   zrmid*sin(zlonm) - 0.5*dxmap

            k = 1
            do 31 j=1,nlat
               if( irep.eq.04 ) then
                  zlatg = gaulat(jsgaus-j+1)*zradi
               else
                  zlatg = (alats + (j-1)*dtheta)*zradi
               endif
               if( prolat.gt.0. ) then
                  if( zlatg.lt.-epspol ) zlatg = -epspol
                  zr = zrlen*cos(zlatg)/(1.+sin(zlatg))
               else
                  if( zlatg.gt.epspol ) zlatg = epspol
                  zr = zrlen*cos(zlatg)/(1.-sin(zlatg))
               endif
               do 30 i=1,nlon
                  zlong = (alonw + (i-1)*dlamda)*zradi - zprolo
                  if( prolat.lt.0. ) zlong = - zlong
                  xgrid(k) = zr*sin(zlong) - zxor + 1.5e-5
                  ygrid(k) = -zr*cos(zlong) - zyor + 1.5e-5
   30          k = k + 1
   31       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return
c
c
c---------------------------------------------------------------------------
c
c    map projection is kulan
c
         else if( maprep.eq.30 ) then

            zmult = rearth/(scale*1.e+6)*100.

            k = 1
            do 41 j=1,nlat
               if( irep.eq.04 ) then
                  zlatg = gaulat(jsgaus-j+1)
               else
                  zlatg = (alats + (j-1)*dtheta)
               endif
               do 40 i=1,nlon
                  zlong = (alonw + (i-1)*dlamda)
                  call kulan(zlatg,zlong,x,y,latmid,lonmid,strch)
                  ygrid(k) = zmult*y + dymap*0.5
                  xgrid(k) = zmult*x + dxmap*0.5
   40          k = k + 1
   41       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return
c
c--------------------------------------------------------------------------
c
c   combination of field and map projections not yet implemented
c
         else
            goto 9000
         endif
c
c**************************************************************************
c
c    input field is in lat/long grid with shifted pole
c
      else if( irep.eq.10 ) then

c-----------------------------------------------------------------------------
c
c    map projection is regular lat/long
c
         if( maprep.eq.0 ) then

            zfac = zradi*rearth/(scale*1.e+6)*100.0
            zyor =  latmid*zfac - 0.5*dymap
            zlomin = lonmid - 0.5*dxmap/zfac
            if( zlomin.lt.-180. ) zlomin = zlomin + 360.
            zxor =  zlomin*zfac
            k = 1
            do 110 j=1,nlat
               zlatl = alats + (j-1)*dtheta
               do 100 i=1,nlon
                  zlonl = alonw + (i-1)*dlamda
                  zlatg = fiprim(zlatl,zlonl,-alatnp)
                  zlong = alampr(zlatl,zlonl,-alatnp)
                  if( zlong.lt.(zlomin-dlamda) ) then
                     zlong = zlong + 360.
                  endif
                  xgrid(k) = zlong*zfac - zxor + 1.5e-5
                  ygrid(k) = zlatg*zfac - zyor + 1.5e-5
  100          k = k + 1
  110       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return
c
c---------------------------------------------------------------------------
c
c    map projection is mercator
c
         else if( maprep.eq.01 ) then

            zprola = prolat*zradi
            zmult = rearth*cos(zprola)/(scale*1.e+6)*100.
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi
            if( zlonm.gt.pi ) zlonm = zlonm - 2.*pi
            zxor = zmult*zlonm - 0.5*dxmap
            zyor = zmult*alog( (1.+sin(zlatm))/cos(zlatm) )
     x               - 0.5*dymap
            if( zxor/zmult.lt.-pi ) then
               zlonm = lonmid*zradi + 2.*pi
               zxor = zmult*zlonm - 0.5*dxmap
            endif
c
            zlongm = zxor/zmult - dlamda*zradi
            k = 1
            do 150 j=1,nlat
               zlatl = (alats + (j-1)*dtheta)
               do 140 i=1,nlon
                  zlonl = (alonw + (i-1)*dlamda)
                  zlatg = fiprim(zlatl,zlonl,-alatnp)*zradi
                  zlong = alampr(zlatl,zlonl,-alatnp)*zradi
                  if( zlatg.lt.-epspol ) zlatg = -epspol
                  if( zlatg.gt.epspol ) zlatg = epspol
                  ygrid(k) = zmult*
     +               alog( (1.+sin(zlatg))/cos(zlatg) ) - zyor
c                 if ( zlong.lt.zlongm ) zlong = zlong + 2.*pi
                  xgrid(k) = zmult*zlong - zxor
  140          k = k + 1
  150       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return
c
c--------------------------------------------------------------------------
c
c    map projection is lat/long with shifted pole
c
         else if( maprep.eq.10 ) then

            zfac = zradi*rearth/(scale*1.e+6)*100.0
            if( ltotal ) then
               zlomin = alonw
               zlamin = alats
               xmin = 1.e-5
               ymin = 1.e-5
               xmax = (alone-alonw)*zfac + 2.e-5
               ymax = (alatn-alats)*zfac + 2.e-5
            else
               zlatg = fiprim(latmid,lonmid,alatnp)
               zlong = alampr(latmid,lonmid,alatnp)
               zlamin = zlatg - 0.5*dymap/zfac
               zlomin = zlong - 0.5*dxmap/zfac
               xmin = 1.e-5
               ymin = 1.e-5
               xmax = dxmap + 2.e-5
               ymax = dymap + 2.e-5
            endif

            zyadd = ( alats - dtheta - zlamin)*zfac
     x         + 1.5e-5
            zymult = dtheta*zfac

            k = 1
            do 200 j=1,nlat
               zy = j*zymult + zyadd
               do 200 i=1,nlon
                  zlon = alonw + float(i-1)*dlamda
                  xgrid(k) = (zlon-zlomin)*zfac
                  ygrid(k) = zy
  200       k = k + 1
            return

c------------------------------------------------------------------------
c
c    map projection is polarstereographic

         else if( maprep.eq.5 ) then

            zprola = prolat*zradi
            zprolo = prolon*zradi
            if( prolat.ge.0. ) then
               zrlen = ( 1. + sin(zprola))*
     +            rearth/(scale*1.e+6)*100.
            else
               zrlen = ( 1. - sin(zprola))*
     +            rearth/(scale*1.e+6)*100.
            endif
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi - zprolo
            if( prolat.ge.0. ) then
               zrmid = zrlen*cos(zlatm)/(1.+sin(zlatm))
            else
               zrmid = zrlen*cos(zlatm)/(1.-sin(zlatm))
               zlonm = - zlonm
            endif
            zyor = - zrmid*cos(zlonm) - 0.5*dymap
            zxor =   zrmid*sin(zlonm) - 0.5*dxmap
            k = 1
            do 310 j=1,nlat
               zlatl = alats + (j-1)*dtheta
               do 300 i=1,nlon
                  zlonl = alonw + (i-1)*dlamda
                  zlatg = fiprim(zlatl,zlonl,-alatnp)*zradi
                  zlong = alampr(zlatl,zlonl,-alatnp)*zradi
     +               - zprolo
                  if( prolat.gt.0. ) then
                     if( zlatg.lt.-epspol ) zlatg = -epspol
                     zr = zrlen*cos(zlatg)/(1.+sin(zlatg))
                  else
                     if( zlatg.gt.epspol) zlatg = epspol
                     zr = zrlen*cos(zlatg)/(1.-sin(zlatg))
                     zlong = - zlong
                  endif
                  xgrid(k) = zr*sin(zlong) - zxor + 1.5e-5
                  ygrid(k) = -zr*cos(zlong) - zyor + 1.5e-5
  300          k = k + 1
  310       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return

c-----------------------------------------------------------------------
c
c    map projection is kulan

         else if( maprep.eq.30 ) then

            zmult = rearth/(scale*1.e+6)*100.

            k = 1
            do 360 j=1,nlat
               zlatl = (alats + (j-1)*dtheta)
               do 350 i=1,nlon
                  zlonl = (alonw + (i-1)*dlamda)
                  zlatg = fiprim(zlatl,zlonl,-alatnp)
                  zlong = alampr(zlatl,zlonl,-alatnp)
                  call kulan(zlatg,zlong,x,y,latmid,lonmid,strch)
                  ygrid(k) = zmult*y + dymap*0.5
                  xgrid(k) = zmult*x + dxmap*0.5
  350          k = k + 1
  360       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return

c-----------------------------------------------------------------------
c
c   combination of field and map projections not yet implemented

         else
            goto 9000
         endif

c**************************************************************************
c
c    input field is in polar stereographic grid
c
      else if( irep.eq.5 ) then

c--------------------------------------------------------------------------
c
c    map projection is polarstereographic
c
         if( maprep.eq.5 ) then

            if( prolat*fig.lt.0. ) then
               write(lulog,*)'mixup of hemispheres prolat=',prolat,
     x                ' and fig=',fig
               return
            endif

            zprola = prolat*zradi
            zprolo = prolon*zradi
            if( prolat.ge.0. ) then
               zrlen = ( 1. + sin(zprola))*rearth/
     x            (scale*1.e+6)*100.
            else
               zrlen = ( 1. - sin(zprola))*rearth
     x            /(scale*1.e+6)*100.
            endif
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi - zprolo
            if( prolat.ge.0. ) then
               zrmid = zrlen*cos(zlatm)/(1.+sin(zlatm))
            else
               zrmid = zrlen*cos(zlatm)/(1.-sin(zlatm))
               zlonm = - zlonm
            endif
            zyor = - zrmid*cos(zlonm) - 0.5*dymap
            zxor =   zrmid*sin(zlonm) - 0.5*dxmap

            zgpla = fig*zradi
            zgplo = ang*zradi
            if( fig.ge.0. ) then
               zrgp = ( 1. + sin(zgpla))*rearth/
     +             (scale*1.e+6)*100.
            else
               zrgp = ( 1. - sin(zgpla))*rearth/
     +             (scale*1.e+6)*100.
            endif
            zmult = ds*zrlen/zrgp/(scale*1.e+6)*100.
            zsalf = sin(zprolo-zgplo)
            zcalf = cos(zprolo-zgplo)
            k = 1
            do 410 j=1,nlat
               zy = (float(j) - ypol)*zmult
               do 400 i=1,nlon
                  zx = (float(i) - xpol)*zmult
                  xgrid(k) = zx*zcalf + zy*zsalf - zxor + 1.e-5
                  ygrid(k) = zy*zcalf - zx*zsalf - zyor + 1.e-5
  400          k = k + 1
  410       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return

c-----------------------------------------------------------------------------
c
c   combination of field and map projections not yet implemented
c
         else
            goto 9000
         endif

c*****************************************************************************
c
c    input field is in rikets koordinater
c
      else if( irep.eq.98 ) then

c-----------------------------------------------------------------------------
c
c    map projection is rikets koordinater
c
         if( maprep.eq.98 ) then

            zfac = 1./(scale*1.e+6)*100.0
            if( ltotal ) then
               zlomin = alonw
               amin = alats
               in = 1.e-5
               in = 1.e-5
               xmax = (alone-alonw)*zfac + 2.e-5
               ymax = (alatn-alats)*zfac + 2.e-5
            else
               call grerik(latmid,lonmid,zlamid,zlomid)
               zlomin = zlomid - 0.5*dxmap/zfac
               zlamin = zlamid - 0.5*dymap/zfac
               xmin = 1.e-5
               ymin = 1.e-5
               xmax = dxmap + 2.e-5
               ymax = dymap + 2.e-5
            endif

            zxadd = ( alonw - dlamda - zlomin)*zfac
     x         + 1.5e-5
            zyadd = ( alats - dtheta - zlamin)*zfac
     x         + 1.5e-5
            zxmult = dlamda*zfac
            zymult = dtheta*zfac

            k = 1
            do 500 j=1,nlat
               zy = j*zymult + zyadd
               do 500 i=1,nlon
                  xgrid(k) = i*zxmult + zxadd
                  ygrid(k) = zy
  500       k = k + 1
            return

c---------------------------------------------------------------------------
c
c    map projection is polarstereographic
c
         else if( maprep.eq.5 ) then

            zprola = prolat*zradi
            zprolo = prolon*zradi
            if( prolat.ge.0. ) then
               zrlen = ( 1. + sin(zprola))*rearth/
     +            (scale*1.e+6)*100.
            else
               zrlen = ( 1. - sin(zprola))*rearth/
     +            (scale*1.e+6)*100.
            endif
            zlatm  = latmid*zradi
            zlonm  = lonmid*zradi - zprolo
            if( prolat.ge.0. ) then
               zrmid = zrlen*cos(zlatm)/(1.+sin(zlatm))
            else
               zrmid = zrlen*cos(zlatm)/(1.-sin(zlatm))
               zlonm = - zlonm
            endif
            zyor = - zrmid*cos(zlonm) - 0.5*dymap
            zxor =   zrmid*sin(zlonm) - 0.5*dxmap
            k = 1
            do 610 j=1,nlat
               zlatl = alats + (j-1)*dtheta
               do 600 i=1,nlon
                  zlonl = alonw + (i-1)*dlamda
                  call rikgre(zlatl,zlonl,zlatg,zlong)
                  zlatg = zlatg*zradi
                  zlong = zlong*zradi - zprolo
                  if( prolat.gt.0. ) then
                     if( zlatg.lt.-epspol ) zlatg = -epspol
                     zr = zrlen*cos(zlatg)/(1.+sin(zlatg))
                  else
                     if( zlatg.gt.epspol) zlatg = epspol
                     zr = zrlen*cos(zlatg)/(1.-sin(zlatg))
                     zlong = - zlong
                  endif
                  xgrid(k) = zr*sin(zlong) - zxor + 1.5e-5
                  ygrid(k) = -zr*cos(zlong) - zyor + 1.5e-5
  600          k = k + 1
  610       continue
            xmin = 1.e-5
            ymin = 1.e-5
            xmax = dxmap + 2.e-5
            ymax = dymap + 2.e-5
            return

c-----------------------------------------------------------------------------
c
c   combination of field and map projections not yet implemented

         else
            goto 9000
         endif

c******************************************************************************
      endif

 9000 write(lulog,*)' plotting in projection ',maprep,
     +   ' from fields in'
      write(lulog,*)' projection ',irep,
     +   ' not yet implemented!!!!!!!'
      return
      end

